#if defined(BI_COMP_BORLANDC) && defined(BI_STD_RTL) // BI_STD_RTL defined only for Builder
#include <condefs.h>
USERC("browse.rc");
#endif

//----------------------------------------------------------------------------
// ObjectWindows
// Copyright (c) 1998 by Bidus Yura, All Rights Reserved
//----------------------------------------------------------------------------
#include <owlext/core.h>
#pragma hdrstop

#include <owl/functor.h>
#include <owl/edit.h>
#include <owl/button.h>
#include <owlext/browse.h>
#include <owlext/dlgapp.h>

////////////////////////////////////////////////////////////////////////////////
#include <owlext/owlxlink.h>  // have to be only once
////////////////////////////////////////////////////////////////////////////////

#include "browse.rh"

OWLEXT_USING_NAMESPACE
	// So we can refer to all OwlExt classes without the prefacing "OwlExt::"

class TBrowseDlg : public TDialog {
	public:
		TBrowseDlg(TWindow* parent);

	protected:
		void SetupWindow();
		void CmAddNewRow();
		void CmDeleteRow();
		void CmHideCode();
		void CmModifyRow();
		void CmShowCode();
		void EvBrowseChange();
		void EvEditChange();

	private:
		TEdit* dfCode;
		TEdit* dfName;
		TEdit* dfSex;
		TEdit* dfAge;
		TBrowse* Browse;
		TButton* pbShowCode;
		TButton* pbHideCode;

		DECLARE_RESPONSE_TABLE(TBrowseDlg);
};

//------------------------------------------------------------------------------
//	TBrowseDlg
//------------------------------------------------------------------------------
#define COL_CODE		100
#define COL_NAME		101
#define COL_EMAIL		102
#define COL_COUNTRY	103
#define COL_SEX			104
#define COL_AGE			105



DEFINE_RESPONSE_TABLE1(TBrowseDlg, TDialog)
	EV_COMMAND(IDC_SHOWCODE, CmShowCode),
	EV_COMMAND(IDC_HIDECODE, CmHideCode),
	EV_COMMAND(IDC_ADD, CmAddNewRow),
	EV_COMMAND(IDC_MODIFY, CmModifyRow),
	EV_COMMAND(IDC_DELETE, CmDeleteRow),
	EV_EN_CHANGE(IDC_CODE, EvEditChange),
	EV_EN_CHANGE(IDC_NAME, EvEditChange),
	EV_EN_CHANGE(IDC_SEX, EvEditChange),
	EV_EN_CHANGE(IDC_AGE, EvEditChange),
	EV_LBN_SELCHANGE(IDC_BROWSE, EvBrowseChange),
	EV_WM_RBUTTONDOWN,
END_RESPONSE_TABLE;

const MAXEDITLEN = 31;

TBrowseDlg::TBrowseDlg(TWindow* parent)
	: TDialog(parent, IDD_BROWSE)
{
	Browse = new TBrowse(this, IDC_BROWSE);

	Browse->AddColumn(COL_CODE, "Code", -1, false);
	Browse->AddColumn(COL_NAME, "Name", 10);
	Browse->AddColumn(COL_SEX,  "Sex",   5);
	Browse->AddColumn(COL_AGE,  "Age");

	Browse->AddStyle(1);
	Browse->SetStyleTextAlignment(1, DT_RIGHT);
	Browse->AddStyle(2);
	Browse->SetStyleColor(2, TColor::LtRed, GetSysColor(COLOR_WINDOW));

	Browse->SetDefaultStyle(COL_AGE, 1);

	pbShowCode = new TButton(this, IDC_SHOWCODE);
	pbHideCode = new TButton(this, IDC_HIDECODE);

	dfCode = new TEdit(this, IDC_CODE, MAXEDITLEN);
	dfName = new TEdit(this, IDC_NAME, MAXEDITLEN);
	dfSex  = new TEdit(this, IDC_SEX,  MAXEDITLEN);
	dfAge  = new TEdit(this, IDC_AGE,  MAXEDITLEN);
}

void TBrowseDlg::SetupWindow()
{
	TDialog::SetupWindow();

	Browse->AddRow();
	Browse->SetColumnText(COL_CODE, "SC");
	Browse->SetColumnText(COL_NAME, "Steven Carr");
	Browse->SetColumnText(COL_SEX, "M");
	Browse->SetColumnText(COL_AGE, "?");

	Browse->AddRow();
	Browse->SetColumnText(COL_CODE, "DB");
	Browse->SetColumnText(COL_NAME, "Daniel Berman");
	Browse->SetColumnText(COL_SEX, "Big", 2, -1);
	Browse->SetColumnText(COL_AGE, "27");
}

void TBrowseDlg::CmAddNewRow()
{
	char text[MAXEDITLEN];

	int index = Browse->AddRow();

	dfCode->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_CODE, text);

	dfName->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_NAME, text);

	dfSex->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_SEX, text);

	dfAge->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_AGE, text);

	Browse->SetSelIndex(index);
}

void TBrowseDlg::CmModifyRow()
{
	int index = Browse->GetSelIndex();
	if (index == LB_ERR) {
		MessageBox("No row selected !", GetModule()->GetName(), MB_ICONSTOP | MB_OK);
		return;
	}

	char text[MAXEDITLEN];

	dfCode->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_CODE, text, index);

	dfName->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_NAME, text, index);

	dfSex->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_SEX, text, index);

	dfAge->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_AGE, text, index);
}

void TBrowseDlg::CmDeleteRow()
{
	int index = Browse->GetSelIndex();
	if (Browse->GetSelIndex() == LB_ERR) {
		MessageBox("No row selected !", GetModule()->GetName(), MB_ICONSTOP | MB_OK);
		return;
	}

	Browse->DeleteRow(index);

	dfCode->Clear();
	dfName->Clear();
	dfSex->Clear();
	dfAge->Clear();
}

void TBrowseDlg::CmShowCode()
{
	pbShowCode->EnableWindow(false);
	pbHideCode->EnableWindow(true);
	Browse->ShowColumn(COL_CODE, true);
}

void TBrowseDlg::CmHideCode()
{
	pbShowCode->EnableWindow(true);
	pbHideCode->EnableWindow(false);
	Browse->ShowColumn(COL_CODE, false);
}

void TBrowseDlg::EvBrowseChange()
{
	int index = Browse->GetSelIndex();

	if (index != LB_ERR) {
		__OWL string str;

		Browse->GetColumnText(COL_CODE, str, index);
		dfCode->SetText(str.c_str());

		Browse->GetColumnText(COL_NAME, str, index);
		dfName->SetText(str.c_str());

		Browse->GetColumnText(COL_SEX, str, index);
		dfSex->SetText(str.c_str());

		Browse->GetColumnText(COL_AGE, str, index);
		dfAge->SetText(str.c_str());

		::EnableWindow(GetDlgItem(IDC_ADD), false);
		::EnableWindow(GetDlgItem(IDC_MODIFY), true);
		::EnableWindow(GetDlgItem(IDC_DELETE), true);
	}
}

void TBrowseDlg::EvEditChange()
{
	bool bEnable = dfCode->GetTextLen() || dfName->GetTextLen() ||
						dfSex->GetTextLen() || dfAge->GetTextLen();
	::EnableWindow(GetDlgItem(IDC_ADD), bEnable);
	::EnableWindow(GetDlgItem(IDC_MODIFY), bEnable);
	::EnableWindow(GetDlgItem(IDC_DELETE), Browse->GetSelIndex() != LB_ERR);
}

//==============================================================================
static TDialog* CreateDialogFunctor()
{
	return new TBrowseDlg(0);
}


int
OwlMain(int /*argc*/, char* /*argv*/ [])
{
 return TDialogApp(Functor((DialogAppFunctor*)0, CreateDialogFunctor),
                   _T("Browse dialog")).Run();
}

//==============================================================================
#if defined(BI_COMP_BORLANDC) && defined(BI_STD_RTL) // BI_STD_RTL defined only for Builder
#define WinMain
#endif
